# Copyright 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os

import fixtures
import testtools

from os_refresh_config import os_refresh_config


class TestMain(testtools.TestCase):
    def setUp(self):
        super(TestMain, self).setUp()
        self.useFixture(fixtures.NestedTempfile())
        td = self.useFixture(fixtures.TempDir())
        self.useFixture(
            fixtures.EnvironmentVariable(
                "OS_REFRESH_CONFIG_BASE_DIR",
                td.path))

    def _run_main(self, args=[]):
        self.lockdir = self.useFixture(fixtures.TempDir())
        self.lockfile = os.path.join(self.lockdir.path, 'lock')
        return os_refresh_config.main(argv=['os-refresh-config',
                                      '--lockfile', self.lockfile])

    def test_main(self):
        self.assertEqual(0, self._run_main())
        self.assertTrue(os.path.exists(self.lockfile))
        self.assertEqual(0, len(open(self.lockfile).read()))



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import subprocess
import time

import fixtures
import testtools

script_path = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    '../os_refresh_config.py')


class TestCmd(testtools.TestCase):

    def setUp(self):
        super(TestCmd, self).setUp()
        self.assertTrue(os.path.exists(script_path))
        self.useFixture(fixtures.NestedTempfile())
        self.base_dir = self.useFixture(fixtures.TempDir())
        self.lockdir = self.useFixture(fixtures.TempDir())
        self.lockfile = os.path.join(self.lockdir.path, 'lock')

    def _run_cmd(self, args, env={}, input_str=None):
        subproc = subprocess.Popen(args,
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
                                   env=env)
        stdout, stderr = subproc.communicate(input=input_str)
        return (subproc.returncode,
                stdout.decode('utf-8'),
                stderr.decode('utf-8'))

    def _run_orc(self, *args):
        cmd_env = {
            'OS_REFRESH_CONFIG_BASE_DIR': self.base_dir.path,
            'PATH': os.environ.get('PATH')
        }
        cmd_args = [
            script_path,
            '--lockfile', self.lockfile
        ]
        if args:
            cmd_args.extend(args)
        return self._run_cmd(cmd_args, cmd_env)

    def _write_script(self, phase, name, returncode=0, sleep=0):
        phase_dir_name = '%s.d' % phase
        phase_dir = self.base_dir.join(phase_dir_name)
        if not os.path.exists(phase_dir):
            os.mkdir(phase_dir)
        script_path = self.base_dir.join(phase_dir_name, name)
        script_dict = {
            'name': name,
            'returncode': returncode,
            'sleep': sleep
        }
        with open(script_path, 'w') as f:
            f.write('''#!/bin/sh
echo %(name)s starting
sleep %(sleep)s
echo %(name)s done
exit %(returncode)s
''' % script_dict)
        os.chmod(script_path, 0o755)

    def test_cmd(self):
        returncode, stdout, stderr = self._run_orc()
        self.assertEqual(0, returncode)
        self.assertEqual('', stdout)
        self.assertEqual('', stderr)

    def test_cmd_with_scripts(self):
        self._write_script('pre-configure', '10-pre-first', 0, 0)
        self._write_script('pre-configure', '20-pre-second', 0, 1)
        self._write_script('configure', '10-conf-first', 0, 0)
        self._write_script('configure', '20-conf-second', 0, 1)
        self._write_script('post-configure', '10-post-first', 0, 0)
        self._write_script('post-configure', '20-post-second', 0, 1)
        now = time.time()
        returncode, stdout, stderr = self._run_orc()

        # check run time accounts for the 3 seconds of sleep
        self.assertTrue(time.time() - now >= 3.0)
        self.assertEqual('\n'.join([
            '10-pre-first starting',
            '10-pre-first done',
            '20-pre-second starting',
            '20-pre-second done',
            '10-conf-first starting',
            '10-conf-first done',
            '20-conf-second starting',
            '20-conf-second done',
            '10-post-first starting',
            '10-post-first done',
            '20-post-second starting',
            '20-post-second done',
            '',
        ]), stdout)
        self.assertEqual(0, returncode)

    def test_cmd_with_failure(self):
        self._write_script('pre-configure', '10-pre-first', 0)
        self._write_script('pre-configure', '20-pre-second', 99)
        self._write_script('configure', '10-conf-first', 0)
        returncode, stdout, stderr = self._run_orc()
        self.assertEqual('\n'.join([
            '10-pre-first starting',
            '10-pre-first done',
            '20-pre-second starting',
            '20-pre-second done',
            '',
        ]), stdout)
        self.assertEqual(1, returncode)

    def test_cmd_with_timeout(self):
        self._write_script('pre-configure', '10-pre-first', 0, 5)
        self._write_script('pre-configure', '20-pre-second', 0, 5)
        self._write_script('configure', '10-conf-first', 0, 5)

        now = time.time()
        returncode, stdout, stderr = self._run_orc('--timeout', '2',
                                                   '--log-level', 'DEBUG')
        # check run time accounts for the 2 seconds timeout
        self.assertTrue(time.time() - now >= 2.0)
        self.assertEqual('\n'.join([
            '10-pre-first starting',
            '',
        ]), stdout)
        self.assertEqual(1, returncode)

    def test_debug(self):
        returncode, stdout, stderr = self._run_orc('--log-level', 'DEBUG')
        self.assertEqual('', stdout)
        self.assertNotEqual('', stderr)
        self.assertEqual(0, returncode)

    def test_print_phases(self):
        returncode, stdout, stderr = self._run_orc('--print-phases')
        self.assertEqual(
            'pre-configure\tconfigure\tpost-configure\tmigration\n',
            stdout
        )
        self.assertEqual('', stderr)
        self.assertEqual(0, returncode)

    def test_print_base(self):
        returncode, stdout, stderr = self._run_orc('--print-base')
        self.assertEqual('%s\n' % self.base_dir.path, stdout)
        self.assertEqual('', stderr)
        self.assertEqual(0, returncode)



# Copyright 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from os_refresh_config import os_refresh_config


class TestRefreshConfig(testtools.TestCase):
    def test_default_base_dir(self):
        default = '/usr/libexec/os-refresh-config'
        with mock.patch('os.path.isdir', lambda x: x == default):
            self.assertEqual(default, os_refresh_config.default_base_dir())

    def test_default_base_dir_deprecated(self):
        default = '/opt/stack/os-config-refresh'
        with mock.patch('os.path.isdir', lambda x: x == default):
            self.assertEqual(default, os_refresh_config.default_base_dir())

    def test_default_base_dir_both(self):
        default = '/usr/libexec/os-refresh-config'
        deprecated = '/opt/stack/os-config-refresh'
        with mock.patch('os.path.isdir', lambda x: (x == default or
                                                    x == deprecated)):
            self.assertEqual(default, os_refresh_config.default_base_dir())



#!/usr/bin/env python
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import fcntl
import logging
import os
import signal
import subprocess
import sys
import time

import psutil

OLD_BASE_DIR = '/opt/stack/os-config-refresh'
DEFAULT_BASE_DIR = '/usr/libexec/os-refresh-config'


def default_base_dir():
    """Determine the default base directory path

    If the OS_REFRESH_CONFIG_BASE_DIR environment variable is set,
    use its value.
    Otherwise, prefer the new default path, but still allow the old one for
    backwards compatibility.
    """
    base_dir = os.environ.get('OS_REFRESH_CONFIG_BASE_DIR')
    if base_dir is None:
        # NOTE(bnemec): Prefer the new location, but still allow the old one.
        if os.path.isdir(OLD_BASE_DIR) and not os.path.isdir(DEFAULT_BASE_DIR):
            logging.warning('Base directory %s is deprecated. The recommended '
                            'base directory is %s',
                            OLD_BASE_DIR, DEFAULT_BASE_DIR)
            base_dir = OLD_BASE_DIR
        else:
            base_dir = DEFAULT_BASE_DIR
    return base_dir


BASE_DIR = default_base_dir()

PHASES = ['pre-configure',
          'configure',
          'post-configure',
          'migration']


def timeout():
    p = psutil.Process()
    children = list(p.get_children(recursive=True))
    for child in children:
        child.kill()


def exit(lock, statuscode=0):
    signal.alarm(0)
    if lock:
        lock.truncate(0)
        lock.close()
    return statuscode


def main(argv=sys.argv):
    parser = argparse.ArgumentParser(
        description="""Runs through all of the phases to ensure
        configuration is applied and enabled on a machine. Will exit with
        an error if any phase has a problem. Scripts should not depend on
        eachother having worked properly. Set OS_REFRESH_CONFIG_BASE_DIR
        environment variable to override the default
        """)
    parser.add_argument('--print-base', default=False, action='store_true',
                        help='Print base dir and exit')
    parser.add_argument('--print-phases', default=False, action='store_true',
                        help='Print phases (tab separated) and exit')
    parser.add_argument('--log-level', default='INFO',
                        choices=['ERROR', 'WARN', 'CRITICAL', 'INFO', 'DEBUG'])
    parser.add_argument('--lockfile',
                        default='/var/run/os-refresh-config.lock',
                        help='Lock file to prevent multiple running copies.')
    parser.add_argument('--timeout',
                        type=int,
                        help='Seconds until the current run will be '
                             'terminated.')
    options = parser.parse_args(argv[1:])

    if options.print_base:
        print(BASE_DIR)
        return 0

    if options.print_phases:
        print("\t".join(PHASES))
        return 0

    log = logging.getLogger('os-refresh-config')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel(options.log_level)

    # Keep open (and thus, locked) for duration of program
    lock = open(options.lockfile, 'a')
    try:
        fcntl.flock(lock, fcntl.LOCK_EX | fcntl.LOCK_NB)
    except IOError as e:
        log.error('Could not lock %s. %s' % (options.lockfile, e))
        return e.errno

    lock.truncate(0)
    lock.write("Locked by pid==%d at %s\n" % (os.getpid(), time.localtime()))

    def timeout_handler(signum, frame):
        log.error('Timeout reached: %ss. Sending SIGKILL to all children' %
                  options.timeout)
        timeout()

    if options.timeout:
        signal.signal(signal.SIGALRM, timeout_handler)
        signal.alarm(options.timeout)

    for phase in PHASES:
        phase_dir = os.path.join(BASE_DIR, '%s.d' % phase)
        log.debug('Checking %s' % phase_dir)
        if os.path.exists(phase_dir):
            args = ['dib-run-parts']
            args.append(phase_dir)
            try:
                log.info('Starting phase %s' % phase)
                log.debug('Running %s' % args)
                subprocess.check_call(args, close_fds=True)
                sys.stdout.flush()
                sys.stderr.flush()
                log.info('Completed phase %s' % phase)
            except subprocess.CalledProcessError as e:
                log.error("during %s phase. [%s]\n" % (phase, e))
                error_dir = os.path.join(BASE_DIR, 'error.d')
                if os.path.exists(error_dir):
                    log.info('Calling error handlers.')
                    try:
                        subprocess.call(['dib-run-parts', error_dir])
                    except OSError:
                        pass
                log.error("Aborting...")
                return exit(lock, 1)
        else:
            log.debug('No dir for phase %s' % phase)

    return exit(lock)


if __name__ == '__main__':
    sys.exit(main(sys.argv))
